/**
 * @file  AECommProject.h
 * @brief Haupt-Header-Datei fuer die DLL AECommProject.
 */
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_AE3COMPROJECT_H__9C5686AB_BB8E_4574_9C30_58A46A1A47AB__INCLUDED_)
#define AFX_AE3COMPROJECT_H__9C5686AB_BB8E_4574_9C30_58A46A1A47AB__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifdef _WIN32
#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"		// Hauptsymbole
#endif

// Kommunikationsarray fuer Verbindungen
#include "AEChannelArray.h"
// Versionspruefung
#include "../Common/DFComUtils.h"

#ifdef _WIN32
#include <afxmt.h> // Fuer CCriticalSection


// Fuer XML-RPC
#include "../Common/Communication/xmlrpc/XmlRpc.h"

// Kontainerklassen fuer Prioritaetswarteschlange importieren.
#include <deque>
#include <queue>

typedef std::deque<CNotifyRecord> notify_deque;
typedef std::priority_queue<CNotifyRecord, notify_deque> notify_priority_deque;

class CPriorityDeque : public notify_priority_deque
{
public:
	int CheckFor(CNotifyRecord &nr)
	{
		int i, res = 0;
		for (i=0; i < c.size(); i++) {

			if ((nr.serialNum == c.at(i).serialNum) && (nr.deviceType == c.at(i).deviceType)) {

				res = 1;
				break;
			}
		}

		return res;
	}

};
#endif
/////////////////////////////////////////////////////////////////////////////
// CAECommProjectApp
// Siehe AECommProject.cpp fr die Implementierung dieser Klasse
//

/// Anwendungsobjekt.
#ifdef _WIN32
class CAECommProjectApp : public CWinApp
#else
class CAECommProjectApp
#endif
{
public:
	/// Konstruktor
	CAECommProjectApp();
	static CAECommProjectApp *m_this;

#ifdef _WIN32
// berladungen
	// Vom Klassenassistenten generierte berladungen virtueller Funktionen
	//{{AFX_VIRTUAL(CAECommProjectApp)
	public:
	virtual BOOL InitInstance();
	virtual int ExitInstance();
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CAECommProjectApp)
		// HINWEIS - An dieser Stelle werden Member-Funktionen vom Klassen-Assistenten eingefgt und entfernt.
		//    Innerhalb dieser generierten Quelltextabschnitte NICHTS VERNDERN!
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

public:
	int StartListener();
	int EndListener();
	int StartNotify();
	int EndNotify();

	// CallBack - Funktion zur Meldung von Datensaetzen.
	// Diese Funktion wird von den Einzelnen CChannel4 Kanalklassen genutzt.
	static int CBNotifyRecord(CNotifyRecord &nr)
	{
		CCriticalSection cs;
		cs.Lock();

		// Bereits Eintraeg vom vorhanden?
		if (m_this->m_queue.CheckFor(nr)) {

			TRACE("Skip: %i, %i; %i\n", nr.serialNum, nr.deviceType, m_this->m_queue.size());
			return IV_ERROR_SUCCESS;
		}

		m_this->m_queue.push(nr);
		TRACE("Put: %i, %i; %i\n", nr.serialNum, nr.deviceType, m_this->m_queue.size());
		
		cs.Unlock();

		return IV_ERROR_SUCCESS;
	}

// Main - Funktion fuer Befehlsausfuehrung in Workerthread
protected:
	static unsigned long WINAPI ListenThread(LPVOID lpParam);
	static unsigned long WINAPI NotifyThread(LPVOID lpParam);
	volatile int m_endSignal[2];
	enum t_state {DOWN, RESET, LISTENING, ACCEPTING};
	t_state m_listenThreadState;
	unsigned short m_listenPort;
	HANDLE m_workerThread[2];
	// Socket fuer Listen
	SOCKET m_listenSocket;
#endif
	
// Helper
protected:
	/// Anfrage an Kanalarray stellen.
	bool AEChannelRequest(short channelID);
#ifdef WIN32
	bool Channel4Request(short &channelID);
#endif
	
// Helper
public:
	/// Ermitteln der Kanalklasse aus Array.
	CAEChannel *GetAEChannel(short channelID);
#ifdef WIN32
	CChannel4 *GetChannel4(short &channelID);
	CPriorityDeque m_queue;

	int RecordAvailable(int *deviceType, int *serialNum, int *commType, char *commString, int *commStringLength, int *commValue, int *deviceIDRemote, int *error)
	{
		int res = IV_ERROR_SUCCESS;

		if (m_queue.size()) {
			CCriticalSection cs;
			cs.Lock();
		
			CNotifyRecord &nr = m_queue.top();

			*deviceType = nr.deviceType;
			*serialNum = nr.serialNum;
			*commType = nr.commType;
			strncpy(commString, (char*)nr.commString, *commStringLength);
			*commStringLength = strlen(commString);
			*commValue = nr.commValue;
			*deviceIDRemote = nr.deviceIDRemote;
			
			m_queue.pop();
			cs.Unlock();

		} else {

			res = -IV_ERROR_NO_DATA_AVAILABLE;
		}

		return res;
	}

public:
	int SetNotifyDevice(int type, char *ip, int port, int maxConnections, int state);
	int SetNotifySoftware(int type, char *ip, int port, int retry, int delay, int state);

#endif

// Membervars
protected:
	/// Arrayobjekt fuer Kanalklassen.
	CAEChannelArray m_channelArray;
#ifdef WIN32
	CChannel4Array m_channel4Array;
#endif

public:
	// Versionspruefungsobjekt
	CDFComUtils m_util;
};

/////////////////////////////////////////////////////////////////////////////
extern CAECommProjectApp theApp;

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ fgt unmittelbar vor der vorhergehenden Zeile zustzliche Deklarationen ein.

#endif // !defined(AFX_AE3COMPROJECT_H__9C5686AB_BB8E_4574_9C30_58A46A1A47AB__INCLUDED_)

