/*!
    \file   df_access_control_lists_v1.h
    \author Bernd Ottmann
    \date   21.02.2003
    \brief  Deklarationen für die Listen der Zutrittskontrolle Version 1.
*/
#ifndef DF_ACCESS_CONTROL_LISTS_V1_H
#define DF_ACCESS_CONTROL_LISTS_V1_H

#include "df_defines.h"

#if (DF_API_VERSION >= 0x02000500)


/*! 
    \brief Setzen des Listenbuffers fuer eine Liste.

    Die Listen werden zusaetzlich intern komprimiert.

    \param connectionId Kommunikationskanalnummer.
    \param number Tabellennummer.
    \param recordCount Anzahl der Zeilen, Datensaetze im Block.
    \param size Anzahl Byte im Block. Brechnet sich aus (Anzahl Datensaetzte    Datensatzlaenge).
    \param buffer Zeiger auf Bytearray, welches die Daten enthaelt.
    \param errorNumber Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.

    \return Ergebnis der Ausfuehrung.
    \retval 0 Funktion konnte nicht ausgefuehrt werden.
    \retval 1 Funktion wurde erfolgreich ausgefuehrt.

    \version Ab DLL - Version 2.0.5 enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCMakeEntranceList( int connectionId, int number, int recordCount, int size, unsigned char *buffer, int *errorNumber );


/*! 
    \brief Uebertragen der Listendaten zum Geraet.

    Es wird versucht, die zuvor mit DFCMakeEntranceListe() importierten Listendaten in 
    das Geraet zu uebertragen.

    \param connectionId Kommunikationskanalnummer.
    \param deviceAddress Busnummer des anzusprechenden Geraetes.
    \param number Tabellennummer der zu uebertragenden Liste. (-1 uebertraegt alle importierten)
    \param errorNumber Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.

    \return Ergebnis der Ausfuehrung.
    \retval 0 Funktion konnte nicht ausgefuehrt werden.
    \retval 1 Funktion wurde erfolgreich ausgefuehrt.

    \version Ab DLL - Version 2.0.5 enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCLoadEntranceList( int connectionId, int deviceAddress, int number, int *errorNumber );


/*! 
    \brief Buffer fuer Listendaten loeschen.

    Loescht in der DLL den Buffer fuer Listendaten.

    \param connectionId Kommunikationskanalnummer.
    \param number Listennummer (0 - 3). Bei Angabe von -1 werden alle geloescht.

    \return Keine.

    \version Ab DLL - Version 2.0.5 enthalten.
*/
DF_API_EXTERN void DF_API_CALL DFCClearEntranceListBuffer( int connectionId, int number );

#endif
#endif
