/*!
    \file   df_files.h
    \author Bernd Ottmann
    \date   21.02.2003
    \brief  Deklarationen der Funktionen für das Übertragen und das Bearbeiten von Dateien.
*/
#ifndef DF_FILES_H
#define DF_FILES_H

#include "df_defines.h"

/*! 
    \brief Schreiben einer Setupdatei.

    Die Setupdatei (*.aes; *.ael) wird auf das Gerät übertragen. Danach müssen ggf. die Listen neu übertragen werden. Ebenfalls ggf. die Zutrittslisten oder Timeboylisten.
    Es dürfen sich keine abrufbare Datensätze mehr auf dem Gerät befinden.

    \param[in] connectionId Verbindungs-Id. Diese erhalten Sie ggf. mittels DFCComOpenIV() oder per Active-Mode.
    \param[in] deviceAddress Addresse eines Gerätes im RS485 Bus, andernfalls 0.
    \param[in] filePath Die zu übertragende Setupdatei mit einer der Endungen *.aes oder *.ael.
    \param[out] errorNumber Variable für einen Fehlercode, falls die Ausführung scheitert.

    \return 1 wenn Erfolgreich, andernfalls 0. Weitere Informationen siehe \link errorNumber Fehlernummer. \endlink
*/
DF_API_EXTERN int DF_API_CALL DFCSetupLaden( int connectionId,int deviceAddress, char *filePath, int *errorNumber );

#if (DF_API_VERSION >= 0x02000000)
/*! 
    \brief Lesen einer Setupdatei.

    Die im Gerät enthaltene Setupdatei wird ausgelesen und in der angegebenen Datei gespreichert. 

    \param connectionId Verbindungs-Id. Diese erhalten Sie ggf. mittels DFCComOpenIV() oder per Active-Mode..
    \param deviceAddress Busnummer des anzusprechenden Geraetes.
    \param filePath Zeiger auf die Zeichenfolge, welche den Pfad und/oder Dateinamen enthaelt.
    \param errorNumber Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.

    \return 1 wenn Erfolgreich, andernfalls 0.

    \version Diese Funktion wurde in der Version 02.00.00 hinzugefügt.
*/
DF_API_EXTERN int DF_API_CALL DFCDownload( int connectionId,int deviceAddress, char *filePath, int *errorNumber );

#endif

DF_API_EXTERN int DF_API_CALL DFCModifyStudioFile( char *sourceFilePath, char *destFilePath, char *xmlData, int flags, int *errorNumber );

#if (DF_API_VERSION >= 0x02001800)
/*! 
    \brief Uebertraegt Dateidaten zum Geraet.

    .

    \param connectionId Verbindungs-Id. Diese erhalten Sie ggf. mittels DFCComOpenIV() oder per Active-Mode..
    \param filePath Name mit eventueller Pfadangabe der Datei.
    \param errorNumber Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.

    \return 1 wenn Erfolgreich, andernfalls 0.

    \version Diese Funktion wurde in der Version 02.00.18 hinzugefügt.
*/
DF_API_EXTERN int DF_API_CALL DFCUpload( int connectionId, int deviceAddress, char *filePath, int *errorNumber );
#endif

#if (DF_API_VERSION >= 0x04020300)
DF_API_EXTERN int DF_API_CALL DFCFileUpload( int connectionId, int deviceAddress, int fileType, char *filePath, int flags, int *errorNumber );
DF_API_EXTERN int DF_API_CALL DFCFileDownload( int connectionId, int deviceAddress, int fileType, char *filePath, int flags, int *errorNumber );
#endif

#if (DF_API_VERSION >= 0x04010500)
DF_API_EXTERN int DF_API_CALL DFCUploadModule( int connectionId, int deviceAddress, int fileType, char *filePath, int *errorNumber );
#endif

#endif
