#ifndef _DATAFLASH_H_
#define _DATAFLASH_H_

// Maximale Anzahl der Flashseiten V1 Atmel
#define DATAFLASH_V1_MAX_PAGE  4096
// Groesse einer Flashseite in Byte (davon sind die hinteren 16 Byte fuer interne Zwecke verwendet)
#define DATAFLASH_V1_MAX_PAGE_SIZE 528
// Groesse der Erweiterten Kontrolldaten einer Flashseite
#define DATAFLASH_V1_PAGE_SIZE_EXTENDED 16
// Groesse des Dateninhalts in einer Flashseite
#define DATAFLASH_V1_PAGE_SIZE_DATA (DATAFLASH_V1_MAX_PAGE_SIZE - DATAFLASH_V1_PAGE_SIZE_EXTENDED)
// 2MByte Gesamtspeicher
#define DATAFLASH_V1_SIZE		(DATAFLASH_V1_MAX_PAGE * DATAFLASH_V1_MAX_PAGE_SIZE)

// Maximale Anzahl der Flashseiten V2 Atmel
#define DATAFLASH_V2_MAX_PAGE  8192
// Groesse einer Flashseite in Byte (davon sind die hinteren 16 Byte fuer interne Zwecke verwendet)
#define DATAFLASH_V2_MAX_PAGE_SIZE 528
// Groesse der Erweiterten Kontrolldaten einer Flashseite
#define DATAFLASH_V2_PAGE_SIZE_EXTENDED 16
// Groesse des Dateninhalts in einer Flashseite
#define DATAFLASH_V2_PAGE_SIZE_DATA (DATAFLASH_V2_MAX_PAGE_SIZE - DATAFLASH_V2_PAGE_SIZE_EXTENDED)
// 4MByte Gesamtspeicher
#define DATAFLASH_V2_SIZE		(DATAFLASH_V2_MAX_PAGE * DATAFLASH_V2_MAX_PAGE_SIZE)
 // Entspricht: 4325376 Byte = 4,125 MByte

// Maximale Anzahl der Flashseiten V3 Atmel
#define DATAFLASH_V3_MAX_PAGE  8192
// Groesse einer Flashseite in Byte (davon sind die hinteren 16 Byte fuer interne Zwecke verwendet)
#define DATAFLASH_V3_MAX_PAGE_SIZE 1056
// Groesse der Erweiterten Kontrolldaten einer Flashseite
#define DATAFLASH_V3_PAGE_SIZE_EXTENDED 32
// Groesse des Dateninhalts in einer Flashseite
#define DATAFLASH_V3_PAGE_SIZE_DATA (DATAFLASH_V3_MAX_PAGE_SIZE - DATAFLASH_V3_PAGE_SIZE_EXTENDED)
// 8MByte Gesamtspeicher
#define DATAFLASH_V3_SIZE		(DATAFLASH_V3_MAX_PAGE * DATAFLASH_V3_MAX_PAGE_SIZE)

// Maximale Anzahl der Flashseiten V4 Adesto
#define DATAFLASH_V4_MAX_PAGE  32768
// Groesse einer Flashseite in Byte (davon sind die hinteren 16 Byte fuer interne Zwecke verwendet)
#define DATAFLASH_V4_MAX_PAGE_SIZE 264
// Groesse der Erweiterten Kontrolldaten einer Flashseite
#define DATAFLASH_V4_PAGE_SIZE_EXTENDED 8
// Groesse des Dateninhalts in einer Flashseite
#define DATAFLASH_V4_PAGE_SIZE_DATA (DATAFLASH_V4_MAX_PAGE_SIZE - DATAFLASH_V4_PAGE_SIZE_EXTENDED)
// 8MByte Gesamtspeicher
#define DATAFLASH_V4_SIZE		(DATAFLASH_V4_MAX_PAGE * DATAFLASH_V4_MAX_PAGE_SIZE)

class CDataFlash 
{
public:
	CDataFlash();
	virtual ~CDataFlash();

	// Ruecksetzen
	void Reset();

	// EEPROM
	inline unsigned char *GetMemDirect(unsigned int address) { 
		address -= m_baseAddress;
		if (address >= DATAFLASH_V3_SIZE) 
		{
#ifdef _MSC_VER
//			ASSERT(0);
#endif
			return 0;
		}
		return &m_mem[address]; 
	}
	inline unsigned char GetMem(unsigned int address) { 
		address -= m_baseAddress;
		if (address >= DATAFLASH_V3_SIZE) 
		{
#ifdef _MSC_VER
//			ASSERT(0);
#endif
			return 0;
		}
		return m_mem[address]; 
	}
	inline unsigned int GetMemRange(unsigned int address, unsigned char *value, unsigned int size) { 
		address -= m_baseAddress;
		if ((address + size) >= DATAFLASH_V3_SIZE)
		{
#ifdef _MSC_VER
//			ASSERT(0);
#endif
			return 0;
		}
		unsigned int i;
		for (i=address; i < (address + size); i++) {
		
			value[i - address] = m_mem[i];
		}
		return (i - address);
	}
	inline void SetMem(unsigned int address, unsigned char value) { 
		address -= m_baseAddress;
		if (address >= DATAFLASH_V3_SIZE)
		{
#ifdef _MSC_VER
//			ASSERT(0);
#endif
			return;
		}
		m_mem[address] = value; 
		SetMemUsebit(address + m_baseAddress);
	}
	inline void SetMemRange(unsigned int address, unsigned char *value, unsigned int size) { 
		address -= m_baseAddress;
		if ((address + size) >= DATAFLASH_V3_SIZE)
		{
#ifdef _MSC_VER
//			ASSERT(0);
#endif
			return;
		}
		for (unsigned int i=address; i < (address + size); i++) {
		
			m_mem[i] = value[i - address];
			SetMemUsebit(i + m_baseAddress);
		}
	}

	// Helper
	void SetBaseAddress(unsigned int base) { m_baseAddress = base; };
	unsigned int GetBaseAddress() { return m_baseAddress; };
	void SetMemUsebit(unsigned int address);
	unsigned char GetMemUsebit(unsigned int address);
	unsigned int GetMemFirstAddress();
	unsigned int GetMemLastAddress();

protected:
	unsigned int m_baseAddress;
	unsigned char *m_mem;
	unsigned int *m_memUse;
};

#endif
